self.__precacheManifest = (self.__precacheManifest || []).concat([
  {
    "revision": "54fc803ad4e68876e6308807062d9683",
    "url": "/index.html"
  },
  {
    "revision": "df959a132ebb6c1acbb3",
    "url": "/static/css/2.929389fa.chunk.css"
  },
  {
    "revision": "dcf78776a978a6954044",
    "url": "/static/css/main.8d5084fd.chunk.css"
  },
  {
    "revision": "df959a132ebb6c1acbb3",
    "url": "/static/js/2.b9f0e663.chunk.js"
  },
  {
    "revision": "ca9e56bc4f9848094e5b4e249ca2f146",
    "url": "/static/js/2.b9f0e663.chunk.js.LICENSE.txt"
  },
  {
    "revision": "dcf78776a978a6954044",
    "url": "/static/js/main.0f983eeb.chunk.js"
  },
  {
    "revision": "2d9282ac3e4bee87b5f2",
    "url": "/static/js/runtime-main.ce081a3c.js"
  },
  {
    "revision": "4b8ec5cf962ffb622f6d7c0a1a8f8385",
    "url": "/static/media/logo2.4b8ec5cf.png"
  }
]);